function [titr,numlabel] = weakweak7(CA,CB,volA,volB,varargin)
% Plots a titration curve for titrating a weak acid with a weak base 
%   or vice versa.
% The arguments are: CA,CB,volA,volB,pKAA,pKBB,hhplot = 0 or 1 ,forrev ='f' or 'r',numlabel
% The first four arguments are required.
% The last five arguments are optional, default to -5,-5, 0, 'f', 1
% The numlabel is incremented and returned, so that by default passing it to the next call
% plots the text label one line lower. Set numlabel = 0 to force a new plot.
% Example below makes a new plot and then plots two more data series, with the three
% lines of text one after another.
% >>[titr,numl] = weakweak7(0.15,0.10,125,250,2.75,4.74,1,'f',0);
% >>[titr,numl] = weakweak7(0.15,0.10,125,250,3.75,4.74,1,'f',numl);
% >>[titr,numl] = weakweak7(0.15,0.10,125,250,4.75,4.74,1,'f',numl);
% Outputs a matrix in the titr variable with columns:
% volume of A, volume of B,pH,[HA],[A-],[B],[BH+], buffer capacity
% Jason Kahn U Maryland 10/17/2018  version 1.3
% Usage:
% [titr,numl] = weakweak7(CA,CB,volA,volB,pKAA,pKBB,hhplot,forrev,numl);
% For example:
% [titr,numl] = weakweak7(.1,.05,100,225,4.75,3.5,1,'f',0); % or just
% weakweak7(.75,1.0,300,250,6.15); % for simple forward titration with a strong base
% if hhplot is set to 1, will also plot the pH derived from the
% Henderson-Hasselbalch just assuming complete neutralization to give set
% concentrations of [HA] and [A-] or [B] and [BH+].
% Generates a plot of 
% Titration of a weak acid A with concentration CA and volume volA with a
% weak base B of concentration (CB), from a volume of zero up to volB.
% OR
% Titration of a weak base B with concentration CB and volume volB with a
% weak acid A of concentration (CA), from a volume of zero up to volA.
%
% KAA = 10^(-pKAA) is the acid dissociation constant of A, and KBB = 10^(-pKBB)
% is the base dissociation constant of B. Volume units do not matter (as
% long as they are the same!), typically they would be ml.
% The program graphs the variable volume versus the pH to show a typical titration.
% Save the values returned in titr if you need to do multiple plots with different x
% scales. Otherwise the default is to overlay plots, so to make separate
% plots with different axes, give the "figure" command between calls to
% weakweak6 or call it with numlabel set to 0.
% For example, to overlay curves showing the effect of dilution
% [~,numl] = weakweak7(.1,.03,100,400,4.75,3.5,1,'f',0);
% [~,numl] = weakweak7(.01,.003,100,400,4.75,3.5,1,'f',numl);
% [~,numl] = weakweak7(.001,.0003,100,400,4.75,3.5,1,'f',numl);
% [~,numl] = weakweak7(.0001,.00003,100,400,4.75,3.5,1,'f',numl);
% [~,numl] = weakweak7(.00001,.000003,100,400,4.75,3.5,1,'f',numl);
% and then 
% To plot the distribution of HA, A-, B, BH+ species use these commands:
% >> [titr2,~] = weakweak7(.1,.05,100,225,4.75,3.5,1,'f',0);
% >> figure  % So you don't plot on top of the pH plot
% >> plot(titr2(:,2),titr2(:,4),titr2(:,2),titr2(:,5),titr2(:,2),titr2(:,6),titr2(:,2),titr2(:,7))
% >> legend('HA','A-','B','BH+')
% Implemented a dual-plot option with undocumented argument # 9 that must be 'DualPlot'
% Use varargin to allow leaving out arguments pKAA, pKBB, hhplot, forrev, and numblabel
% Defaults:  
pKAA = -5;  % strong acid
pKBB = -5;  % strong base
hhplot = 0; % No H-H plot
forrev = 'f'; % Titrate acid with base ( = forward), vs. base with acid (reverse)
numlabel = 1; % Start at top of an existing plot if there is one
optargs(1:5) = {pKAA,pKBB,hhplot,forrev,numlabel};
% Change default values if user provides them
%numrequired = 0;
numpossible = 7;
numvarargs = length(varargin);
% if numvarargs < numrequired
%     error('weakweak6:NotEnoughInputs', ...
%         'weakweak6 requires at least %d inputs',numrequired);
% end
if numvarargs > numpossible
    error('weakweak6:TooManyInputs', ...
        'weakweak7 can handle at most %d optional inputs',numpossible);
end

% now put these defaults into the valuesToUse cell array, 
% and overwrite the ones specified in varargin.
optargs(1:numvarargs) = varargin;
[pKAA,pKBB,hhplot,forrev,numlabel] = optargs{1:5};
if (forrev == 'f' || forrev == 'F')
        forrev = 'f';
        titrcol = 2;
elseif (forrev == 'r' || forrev == 'R')
        forrev = 'r';
        titrcol = 1;
else
    error('weakweak7:ConfusingDirection', ...
        'weakweak7: please enter ''f'' or ''r'' for direction');
end
if (numvarargs >= 6)
    try
    if (optargs{6} == 'DualPlot')    
        dualp = true;
%     if(numlabel ~= 0)
%         disp('Cannot do multiple dual plots yet.')
%         disp('Creating new figure.')
%         numlabel = 0;
%     end
    end
    catch
        disp(['Cannot parse ',optargs{6},' exiting.'])
        return
    end
else
    dualp = false;
end
%
KW = 1e-14;
KAA = 10^(-pKAA);
KBB = 10^(-pKBB);
KWoKB = KW/KBB;
%hhplot = 0; Should install this as a varargin etc.
npts = 301;  % This is a minimum number, probably must be > 200
%
% First make an array of volumes for the titrant from 0 up to the specified
% volA or VolB
% To deal with forward or reverse titrations, just essentially duplicate
% everythign with labels switched. There is probably a more elgant way to
% do this.
if (forrev == 'f')
% Figure out where the equivalence point is and make points denser in its vicinity. 
eqpt = volA*CA/CB;
disp(['Equiv pt at ',num2str(eqpt)])
if volB > 1.03*eqpt   
    Bvol = [ 0 : (volB-.06*eqpt)/(npts-101) : 0.97*eqpt-.06*eqpt/100 ,...
        0.97*eqpt : .06*eqpt/100 : eqpt-.06*eqpt/100 ,...
        eqpt-.06*eqpt/100+eqpt*2e-5 : eqpt*2e-5 : eqpt+.06*eqpt/100-eqpt*2e-5,...
        eqpt+.06*eqpt/100:.06*eqpt/100:1.03*eqpt ,...
        1.03*eqpt+.06*eqpt/100 : (volB-.06*eqpt)/(npts-101) : volB ];
    if Bvol(end) ~= volB
        Bvol = [Bvol volB];  % For the axes to plot neatly want the fixed end point
    end
    npts = length(Bvol);
    disp(['New npts ', num2str(npts)])
else
    Bvol = 0 : volB/(npts-1) : volB;
end
    Avol = zeros(1,npts);
    Avol(1:npts) = volA;
end
% and the same for reverse
if (forrev == 'r')
% Figure out where the equivalence point is and make points denser in its vicinity. 
eqpt = volB*CB/CA;
disp(['Equiv pt at ',num2str(eqpt)])
if volA > 1.03*eqpt   
    Avol = [ 0 : (volA-.06*eqpt)/(npts-101) : 0.97*eqpt-.06*eqpt/100 ,...
        0.97*eqpt : .06*eqpt/100 : eqpt-.06*eqpt/100 ,...
        eqpt-.06*eqpt/100+eqpt*2e-5 : eqpt*2e-5 : eqpt+.06*eqpt/100-eqpt*2e-5,...
        eqpt+.06*eqpt/100:.06*eqpt/100:1.03*eqpt ,...
        1.03*eqpt+.06*eqpt/100 : (volA-.06*eqpt)/(npts-101) : volA ];
    if Avol(end) ~= volA
        Avol = [Avol volA];  % For the axes to plot neatly want the fixed end point
    end
    npts = length(Avol);
    disp(['New npts ', num2str(npts)])
else
    Avol = 0 : volA/(npts-1) : volA;
end
    Bvol = zeros(1,npts);
    Bvol(1:npts) = volB;
end

% But at this point we just need the concentrations

Bconcs = CB*Bvol./(Avol+Bvol);
Aconcs = CA*Avol./(Avol+Bvol);
if (hhplot == 1)  % Plot the result of just plugging in to HH equation
    % Can't use the first point because it will take the log of 0, but for
    % some reason it did not complain??
    if (forrev == 'f')
        Hhmax = find(Bvol < eqpt,1,'last');
        Hhph = pKAA+log10(Bconcs(2:Hhmax)./(Aconcs(2:Hhmax)-Bconcs(2:Hhmax)));
    else
        Hhmax = find(Avol < eqpt,1,'last');
        Hhph = 14 - pKBB - log10(Aconcs(2:Hhmax)./(Bconcs(2:Hhmax)-Aconcs(2:Hhmax)));
    end
end
proton = zeros(npts,1);
HA = zeros(npts,1);
Aminus = zeros(npts,1);
BaseB = zeros(npts,1);
BHplus =  zeros(npts,1);
pH = zeros(npts,1);
Bufcap = zeros(npts,1);
for i=1:npts
    a1 = 1;
    a2 = Bconcs(i) + KAA + KWoKB;
    a3 = KAA*(Bconcs(i) - Aconcs(i)) + KAA*KWoKB - KW;
    a4 = -Aconcs(i)*KAA*KWoKB - KW*(KAA + KWoKB);
    a5 = -KW*KAA*KWoKB;
    poly = [a1 a2 a3 a4 a5];
    rpoly = roots(poly);   % This is where all the action happens
    proton(i) = rpoly(abs(rpoly > 0) & imag(rpoly) == 0); % Choose the positive real root
    pH(i) = -log10(proton(i));
    % The HA calculation may be wrong in earlier versions?
    HA(i) = (Aconcs(i).*proton(i))./(KAA+proton(i));
    Aminus(i) = Aconcs(i)-HA(i);
    BaseB(i) = Bconcs(i)*(KW/(KBB*proton(i)))/(1+KW/(KBB*proton(i)));
    BHplus(i) = Bconcs(i)-BaseB(i);
    if i > 2
        Bufcap(i-1) = 0.5*((Bconcs(i)-Bconcs(i-1))/(pH(i)-pH(i-1)) ...
            + ((Bconcs(i-1)-Bconcs(i-2))/(pH(i-1)-pH(i-2))));
    end
end
% The buffer capacity calculation is dubious for a weak base titrant.
Bufcap(1) = (Bconcs(2)-Bconcs(1))/(pH(2)-pH(1));
Bufcap(npts) = (Bconcs(npts)-Bconcs(npts-1))/(pH(npts)-pH(npts-1));
%
titr = [Avol' Bvol' pH HA Aminus BaseB BHplus Bufcap];  % Returns a matrix with columns as shown
% The rest is plotting the pH vs. the added volume
%figure  % Uncomment this line to prevent overlaying plots, which is now
%the default. Should have an optional argument to control this from the
%command line.
if(numlabel == 0)
    figure
end
if dualp
	subplot(2,1,1)
end
hold on
box on
symplot = plot(titr(:,titrcol),titr(:,3),'LineWidth',2);
mc = symplot.Color;
plot(titr(:,titrcol),titr(:,3),'o','Color',mc);   % to see points
if forrev == 'f'
    if (hhplot ==1)
        plot(Bvol(2:Hhmax),Hhph,'b-')
    end
    axis([0 max(Bvol) 0 14])    % So different titrations can be compared
    xlimit = xlim;
    if volB > 1.03*eqpt
        xinterval = eqpt/4;
        numxticks = (xlimit(2)- xlimit(1))/xinterval;
    else
        numxticks = 8;
    end
    xtickformat('%.1f')
    set(gca,'FontSize',14,'XTick',xlimit(1) : (xlimit(2)- xlimit(1))/numxticks...
        : xlimit(2))
    title('Acid-Base Titration','FontWeight','bold','FontSize',16)
    xlabel('Added volume of Base','FontWeight','bold','FontSize',14);
    ylabel('pH','FontWeight','bold','FontSize',14);
    text(0.10*max(Bvol),13.5-numlabel*0.75,...
        ['pKa(A) = ',num2str(pKAA),...
        '  pKb(B) = ',num2str(pKBB), ...
        '  CA = ',num2str(CA),' M ',...
        '  CB = ',num2str(CB),' M ',...
        '  Init. vol of A = ',num2str(volA)],...
        'Fontsize',14,'FontWeight','bold','Color',mc)
elseif forrev == 'r'
    if (hhplot ==1)
        plot(Avol(2:Hhmax),Hhph,'b-')
    end
    axis([0 max(Avol) 0 14])    % So different titrations can be compared
    xlimit = xlim;
    if volA > 1.03*eqpt
        xinterval = eqpt/4;
        numxticks = (xlimit(2)- xlimit(1))/xinterval;
    else
        numxticks = 8;
    end
    xtickformat('%.1f')
    set(gca,'FontSize',14,'XTick',xlimit(1) : (xlimit(2)- xlimit(1))/numxticks...
        : xlimit(2))
    title('Base-Acid Titration','FontWeight','bold','FontSize',16)
    xlabel('Added volume of Acid','FontWeight','bold','FontSize',14);
    ylabel('pH','FontWeight','bold','FontSize',14);
    text(0.10*max(Bvol),13.50-0.75*numlabel,...
        ['pKa(A) = ',num2str(pKAA),...
        '  pKb(B) = ',num2str(pKBB), ...
        '  CA = ',num2str(CA),' M ',...
        '  CB = ',num2str(CB),' M ',...
        '  Init. vol of B = ',num2str(volB)],...
        'Fontsize',14,'FontWeight','bold','Color',mc)
end
if dualp == true
    subplot(2,1,2)
    hold on
    box on
    title('Speciation','FontWeight','bold','FontSize',16)
    yyaxis left
    plot(titr(:,2),titr(:,4),titr(:,2),titr(:,5),titr(:,2),titr(:,6),titr(:,2),titr(:,7),'LineWidth',2,'Color',mc,'Marker','none')
    xlim(xlimit);
    xlabel('Added volume of Base','FontWeight','bold','FontSize',14);
    ylabel('Concentrations','FontWeight','bold','FontSize',14);
    yyaxis right
%hold on
    plot(titr(:,2),titr(:,8))
    ylabel('Buffer Capacity','FontWeight','bold','FontSize',14);
	lgd = legend('[HA]','[A-]','[B]','[BH+]','Buffer Cap');
    lgd.FontSize = 14;
else
    numlabel = numlabel + 1;
end
% uncomment line below to save (should rewrite to use export_fig).
%print -dpng -r200
return
